@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: client.c,v 1.38 2005/11/04 09:56:14 evertonm Exp $ */
+/* $Id: client.c,v 1.39 2005/12/20 09:36:09 evertonm Exp $ */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -60,8 +60,8 @@ static void schedule_stat_interval(int sd)
   session->tv_interval.tv_sec += session->stat_interval;
 
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_interval,
-				on_tcp_interval, pipe);
+                                session->tv_interval,
+                                on_tcp_interval, pipe);
 }
 
 static void *on_tcp_read(oop_source *src, int sd,
@@ -84,14 +84,14 @@ static void *on_tcp_read(oop_source *src, int sd,
     if (rd) {
       switch (errno) {
       case EINTR:
-	fprintf(stderr, "read: EINTR on TCP socket %d\n", sd);
-	return OOP_CONTINUE;
+        fprintf(stderr, "read: EINTR on TCP socket %d\n", sd);
+        return OOP_CONTINUE;
       case EAGAIN:
-	fprintf(stderr, "read: EAGAIN on TCP socket %d\n", sd);
-	return OOP_CONTINUE;
+        fprintf(stderr, "read: EAGAIN on TCP socket %d\n", sd);
+        return OOP_CONTINUE;
       default:
-	fprintf(stderr, "read: errno=%d: %s\n",
-		errno, strerror(errno));
+        fprintf(stderr, "read: errno=%d: %s\n",
+                errno, strerror(errno));
       }
     }
 
@@ -110,6 +110,9 @@ static void *on_tcp_read(oop_source *src, int sd,
 
   session = &pipe->session;
 
+  /* modifies 'session->seed' */
+  tcp_check_data(sd, session, buf, rd);
+
   nepim_session_read_add(session, rd);
 
   return OOP_CONTINUE;
@@ -122,6 +125,7 @@ static void *on_tcp_write(oop_source *src, int sd,
   int wr;
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
+  unsigned old_seed;
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
@@ -130,8 +134,13 @@ static void *on_tcp_write(oop_source *src, int sd,
 
   assert(sizeof(buf) == nepim_global.tcp_write_size);
 
+  old_seed = session->seed;
+  /* modifies 'session->seed' */
+  fill_packet_data(session, buf, 0, nepim_global.tcp_write_size);
+
   wr = write(sd, buf, nepim_global.tcp_write_size);
   if (wr < 0) {
+    session->seed = old_seed;   /* rewind seed */
     switch (errno) {
     case EINTR:
       fprintf(stderr, "write: EINTR on TCP socket %d\n", sd);
@@ -153,6 +162,12 @@ static void *on_tcp_write(oop_source *src, int sd,
 
     return OOP_CONTINUE;
   }
+  if (wr < nepim_global.tcp_write_size) {
+      /* rewind seed to use exactly 'wr' bytes */
+      session->seed = old_seed;
+      /* modifies 'session->seed' */
+      fill_packet_data(session, buf, 0, wr);
+  }
 
   assert(wr >= 0);
   assert(wr <= sizeof(buf));
@@ -163,13 +178,14 @@ static void *on_tcp_write(oop_source *src, int sd,
 }
 
 static void *on_tcp_rate_write(oop_source *src, int sd,
-			       oop_event event, void *user)
+                               oop_event event, void *user)
 {
   char buf[nepim_global.tcp_write_size];
   int wr;
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
   int to_write;
+  unsigned old_seed;
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
@@ -205,8 +221,13 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   assert(to_write >= 0);
   assert(to_write <= sizeof(buf));
   
+  old_seed = session->seed;
+  /* modifies 'session->seed' */
+  fill_packet_data(session, buf, 0, to_write);
+
   wr = write(sd, buf, to_write);
   if (wr < 0) {
+    session->seed = old_seed;   /* rewind seed */
     switch (errno) {
     case EINTR:
       fprintf(stderr, "rate_write: EINTR on TCP socket %d\n", sd);
@@ -228,6 +249,12 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
     return OOP_CONTINUE;
   }
+  if (wr < to_write) {
+      /* rewind seed to use exactly 'wr' bytes */
+      session->seed = old_seed;
+      /* modifies 'session->seed' */
+      fill_packet_data(session, buf, 0, wr);
+  }
 
   assert(wr >= 0);
   assert(wr <= sizeof(buf));
@@ -247,12 +274,12 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
     
     /* stop writing */
     nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				    sd, OOP_WRITE);
+                                    sd, OOP_WRITE);
 
     /* schedule for next time saved by on_tcp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_send_rate,
-				  on_tcp_rate_delay, pipe);
+                                  session->tv_send_rate,
+                                  on_tcp_rate_delay, pipe);
   }
 
   return OOP_CONTINUE;
@@ -284,9 +311,9 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 
   /* calculate bytes to be written from rate */
   session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
-					    session->write_delay);
+                                            session->write_delay);
   session->pkt_remaining = nepim_pps2packets(session->max_pkt_rate,
-					     session->write_delay);
+                                             session->write_delay);
 
 #ifndef NDEBUG
  {
@@ -301,8 +328,8 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 
   /* start to write */
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
-			      pipe->sd, OOP_WRITE,
-			      on_tcp_rate_write, pipe);
+                              pipe->sd, OOP_WRITE,
+                              on_tcp_rate_write, pipe);
 
   return OOP_CONTINUE;
 }
@@ -315,13 +342,13 @@ static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
   assert(timercmp(&tv, &session->tv_duration, ==));
 
   nepim_pipe_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
-		  session->byte_total_recv,
-		  session->byte_total_sent,
-		  session->test_duration,
-		  session->tv_start.tv_sec,
-		  session->test_duration,
-		  session->total_reads,
-		  session->total_writes);
+                  session->byte_total_recv,
+                  session->byte_total_sent,
+                  session->test_duration,
+                  session->tv_start.tv_sec,
+                  session->test_duration,
+                  session->total_reads,
+                  session->total_writes);
 
   session->byte_total_recv = 0;
   session->byte_total_sent = 0;
@@ -342,13 +369,13 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
   assert(timercmp(&tv, &session->tv_interval, ==));
 
   nepim_pipe_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
-		  session->byte_interval_recv,
-		  session->byte_interval_sent,
-		  session->stat_interval,
-		  session->tv_start.tv_sec,
-		  session->test_duration,
-		  session->interval_reads,
-		  session->interval_writes);
+                  session->byte_interval_recv,
+                  session->byte_interval_sent,
+                  session->stat_interval,
+                  session->tv_start.tv_sec,
+                  session->test_duration,
+                  session->interval_reads,
+                  session->interval_writes);
 
   session->byte_interval_recv = 0;
   session->byte_interval_sent = 0;
@@ -377,6 +404,11 @@ static int greet(int sd)
   opt.write_delay           = nepim_global.write_delay;
   opt.udp_keepalive_send    = -1;
   opt.udp_keepalive_require = -1;
+  opt.seed                  = nepim_global.seed;
+  opt.verify_data           = nepim_global.verify_data;
+  opt.random_fill           = nepim_global.random_fill;
+  opt.fill_byte             = nepim_global.fill_byte;
+  opt.pause_duration        = nepim_global.pause_duration;
 
   if (nepim_global.password) {
     opt.password_buf        = nepim_global.password;
@@ -410,20 +442,20 @@ static void tcp_pipe_start(int sd)
   session = &pipe->session;
 
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
-			      sd, OOP_READ,
-			      on_tcp_read, pipe);
+                              sd, OOP_READ,
+                              on_tcp_read, pipe);
 
   if (session->must_send) {
     if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_send_rate,
-				    on_tcp_rate_delay, pipe);
+                                    session->tv_send_rate,
+                                    on_tcp_rate_delay, pipe);
     }
     else {
       nepim_global.oop_src->on_fd(nepim_global.oop_src,
-				  sd, OOP_WRITE,
-				  on_tcp_write, pipe);
+                                  sd, OOP_WRITE,
+                                  on_tcp_write, pipe);
     }
   }
 
@@ -434,14 +466,40 @@ static void tcp_pipe_start(int sd)
   }
   session->tv_duration.tv_sec += session->test_duration;
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_duration,
-				on_tcp_duration, pipe);
+                                session->tv_duration,
+                                on_tcp_duration, pipe);
 
   nepim_sock_show_opt(stderr, sd);
 
   schedule_stat_interval(sd);
 }
 
+static void *on_tcp_pipe_start(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_pipe_t *pipe = user;
+
+  tcp_pipe_start(pipe->sd);
+  return OOP_CONTINUE;
+}
+
+static void schedule_tcp_pipe_start(int sd)
+{
+    nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+    nepim_session_t *session;
+    struct timeval pause;
+
+    session = &pipe->session;
+
+    {
+        int result = gettimeofday(&pause, 0);
+        assert(!result);
+    }
+    pause.tv_sec += session->pause_duration;
+    nepim_global.oop_src->on_time(nepim_global.oop_src,
+                                  pause,
+                                  on_tcp_pipe_start, pipe);
+}
+
 static void tcp_pipe_stop(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
@@ -451,32 +509,32 @@ static void tcp_pipe_stop(int sd)
   session = &pipe->session;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_READ);
+                                  sd, OOP_READ);
 
   if (session->must_send) {
     if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       /* stop current writing, if any */
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				      sd, OOP_WRITE);
+                                      sd, OOP_WRITE);
 
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_send_rate,
-					on_tcp_rate_delay, pipe);
+                                        session->tv_send_rate,
+                                        on_tcp_rate_delay, pipe);
     }
     else {
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				      sd, OOP_WRITE);
+                                      sd, OOP_WRITE);
     }
   }
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    session->tv_duration,
-				    on_tcp_duration, pipe);
+                                    session->tv_duration,
+                                    on_tcp_duration, pipe);
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    session->tv_interval,
-				    on_tcp_interval, pipe);
+                                    session->tv_interval,
+                                    on_tcp_interval, pipe);
 
   nepim_pipe_set_del(&pipes, sd);
 
@@ -501,15 +559,15 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
   hints.ai_canonname = 0;
 
   fprintf(stderr, 
-	  "TCP socket solving %s,%s\n",
-	  hostname, portname);
+          "TCP socket solving %s,%s\n",
+          hostname, portname);
 
   result = getaddrinfo(hostname, portname,
-		       &hints, &ai_res);
+                       &hints, &ai_res);
   if (result) {
     fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
             __PRETTY_FUNCTION__, hostname, portname,
-	    gai_strerror(result));
+            gai_strerror(result));
     return;
   }
 
@@ -517,8 +575,8 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
     int sd;
 
     fprintf(stderr, 
-	    "TCP socket trying %s,%s\n",
-	    hostname, portname);
+            "TCP socket trying %s,%s\n",
+            hostname, portname);
 
     if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
       continue;
@@ -527,31 +585,31 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
       continue;
 
     sd = nepim_connect_client_socket(ai->ai_addr, ai->ai_addrlen,
-				     ai->ai_family, ai->ai_socktype, 
-				     ai->ai_protocol,
-				     nepim_global.pmtu_mode,
-				     nepim_global.ttl,
-				     nepim_global.win_recv,
-				     nepim_global.win_send);
+                                     ai->ai_family, ai->ai_socktype, 
+                                     ai->ai_protocol,
+                                     nepim_global.pmtu_mode,
+                                     nepim_global.ttl,
+                                     nepim_global.win_recv,
+                                     nepim_global.win_send);
     if (sd < 0) {
       fprintf(stderr, 
-	      "could not connect TCP socket to %s,%s: %d\n",
-	      hostname, portname, sd);
+              "could not connect TCP socket to %s,%s: %d\n",
+              hostname, portname, sd);
       continue;
     }
 
     fprintf(stderr, 
-	    "%d: TCP socket connected to: "
-	    "host=%s,%s len=%d family=%d type=%d proto=%d\n",
-	    sd, hostname, portname, 
-	    ai->ai_addrlen, ai->ai_family, 
-	    ai->ai_socktype, ai->ai_protocol);
+            "%d: TCP socket connected to: "
+            "host=%s,%s len=%d family=%d type=%d proto=%d\n",
+            sd, hostname, portname, 
+            ai->ai_addrlen, ai->ai_family, 
+            ai->ai_socktype, ai->ai_protocol);
 
     {
       nepim_greet_t opt;
 
       opt.must_send             = nepim_global.duplex_mode ||
-	nepim_global.simplex_client_send;
+        nepim_global.simplex_client_send;
       opt.bit_rate              = nepim_global.bit_rate;
       opt.pkt_rate              = nepim_global.pkt_rate;
       opt.stat_interval         = nepim_global.stat_interval;
@@ -559,6 +617,11 @@ static void spawn_one_tcp_client(const char *hostname, const char *portname)
       opt.write_delay           = nepim_global.write_delay;
       opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
       opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
+      opt.seed                  = nepim_global.seed;
+      opt.verify_data           = nepim_global.verify_data;
+      opt.random_fill           = nepim_global.random_fill;
+      opt.fill_byte             = nepim_global.fill_byte;
+      opt.pause_duration        = nepim_global.pause_duration;
 
       nepim_pipe_set_add(&pipes, sd, ai->ai_addr, ai->ai_addrlen, &opt);
     }
@@ -582,8 +645,8 @@ static void parse_hosts(const char *host_list)
     /* get hostname */
     if (addr_list_get(host_list, j, hostname, sizeof(hostname))) {
       fprintf(stderr, 
-	      "%s: failure parsing address %d/%d from list: %s\n",
-	      nepim_global.prog_name, j, size, host_list);
+              "%s: failure parsing address %d/%d from list: %s\n",
+              nepim_global.prog_name, j, size, host_list);
       continue;
     }
 
@@ -613,13 +676,13 @@ static void spawn_tcp_clients(const char *host_list)
       continue;
 
     nepim_sock_dump_addr(peer, sizeof(peer),
-			 (const struct sockaddr *) &pipe->session.remote);
+                         (const struct sockaddr *) &pipe->session.remote);
 
     if (greet(i)) {
       fprintf(stderr,
-	      "%d: could not greet %s,%d\n",
-	      i, peer, 
-	      nepim_sock_get_port((const struct sockaddr *) &pipe->session.remote));
+              "%d: could not greet %s,%d\n",
+              i, peer, 
+              nepim_sock_get_port((const struct sockaddr *) &pipe->session.remote));
       
       nepim_pipe_set_del(&pipes, i);
       close(i);
@@ -628,11 +691,11 @@ static void spawn_tcp_clients(const char *host_list)
     }
 
     fprintf(stderr, 
-	    "%d: greetings sent to %s,%d\n",
-	    i, peer, 
-	    nepim_sock_get_port((const struct sockaddr *) &pipe->session.remote));
-    
-    tcp_pipe_start(i);
+            "%d: greetings sent to %s,%d\n",
+            i, peer, 
+            nepim_sock_get_port((const struct sockaddr *) &pipe->session.remote));
+
+    schedule_tcp_pipe_start(i);
   }
 
 }
