@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.33 2005/12/20 09:36:09 evertonm Exp $
+  $Id: main.c,v 1.34 2006/03/10 18:56:51 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -102,6 +102,7 @@ static void usage(FILE *out, const char *prog_name)
           "client-only options:\n"
           "  -u             UDP mode (defaults to TCP)\n"
           "  -s             client-send simplex mode (defaults to server-send)\n"
+	  "  -F             force sending out UDP regardless of server\n"
           "  -d             duplex traffic (defaults to simplex)\n"
           "  -M             enable multicast-compatible options\n"
           "  -c host-list   client mode (defaults to server mode)\n"
@@ -433,6 +434,15 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-F")) {
+      nepim_global.udp_mode                = 1; /* -u (UDP) */
+      nepim_global.simplex_client_send     = 1; /* -s (client-send) */
+      nepim_global.udp_keepalive_require   = 0; /* boolean */
+      nepim_global.udp_require_greet_reply = 0; /* boolean */
+      nepim_global.max_greetings           = 0; /* counter */
+      continue;
+    }
+
     if (!strcmp(arg, "-r")) {
       int bytes;
       ++i;
